import logging
from typing import Tuple
import os
import urllib
import urllib.parse
import os
import json
import builtins
import mlflow

def _get_infinstor_dir() -> str:
    if 'INFINSTOR_TOKEN_FILE_DIR' in os.environ:
        return os.environ['INFINSTOR_TOKEN_FILE_DIR']
    else:
        if 'MLFLOW_PARALLELS_URI' in os.environ:
            return os.path.join(os.path.expanduser("~"), ".mlflow-parallels")
        else:
            return os.path.join(os.path.expanduser("~"), ".infinstor")
    
def read_infinstor_config_json_key(jsonkey:str):
    keyval = None
    config_json_path = os.path.join(_get_infinstor_dir(),'config.json')
    if os.path.exists(config_json_path): 
        with open(config_json_path, 'r') as fh:
            config_json:dict = json.load(fh)
            keyval = config_json.get(jsonkey, None)
    return keyval

def get_log_level_from_config_json(module_name:str) -> int:
    """
    Get the loglevel (integer) that correpsonds to the specified module_name, by looking into ~/.infinstor/config.json
    """
    
    loglevel_str:str = read_infinstor_config_json_key('loglevel.' + module_name)
    
    loglevel_int = logging.INFO
    # if config.json has loglevel defined for the specified module    
    if loglevel_str:
        loglevel_int:int = getattr(logging, loglevel_str.upper(), None)
    
    return loglevel_int

def _get_logger(name:str) -> logging.Logger:
    loglevel_int:int = get_log_level_from_config_json(name)
    logging.basicConfig(level=loglevel_int, format="%(asctime)s - %(process)d - %(name)s - %(levelname)s - %(message)s", force=True)
    logger = logging.getLogger(name)
    if loglevel_int:
        logger.setLevel(loglevel_int)
        
    if loglevel_int == logging.DEBUG:
        from http.client import HTTPConnection
        HTTPConnection.debuglevel = 1
    
    return logger

# set the root logger to the loglevel specified in config.json
_root_logger:logging.Logger = _get_logger("root")
_logger:logging.Logger = _get_logger(__name__)

def set_builtins_with_serverside_config(server_config:dict):
    builtins.clientid = server_config.get('clientid', "default_unknown")
    builtins.appclientid = server_config.get('appclientid', "default_unknown")
    builtins.mlflowserver = server_config.get('mlflowserver', "default_unknown")
    builtins.mlflowuiserver = server_config.get('mlflowuiserver', "default_unknown")
    builtins.mlflowstaticserver = server_config.get('mlflowstaticserver', "default_unknown")
    builtins.apiserver = server_config.get('apiserver', "default_unknown")
    builtins.serviceserver = server_config.get('serviceserver', "default_unknown")
    builtins.service = server_config.get('service', "default_unknown")
    builtins.region = server_config.get('region', "default_unknown")
    # server_config (from server) has the string 'true' or 'false' and not boolean True or False
    builtins.isexternalauth = server_config.get('isexternalauth', False)

def _validate_infin_mlflow_tracking_uri_exit(tracking_uri=None, do_exit=False) -> Tuple[str, urllib.parse.ParseResult]:
    """
    attemps to get the MLFLOW_TRACKING_URI from one of these 3 locations:  mlflow.get_tracking_uri(), MlflowClient(tracking_uri=xxxx) and MLFLOW_TRACKING_URI env variable.  And validates if the scheme of the URI is 'infinstor'.  If not, exit()s with an error message

    _extended_summary_

    Args:
        tracking_uri (_type_, optional): The tracking_uri as a string. Will be parsed to ascertain if the scheme is infinstor. Defaults to None.

    Returns:
        Tuple[str, urllib.parse.ParseResult]: A tuple of infinstor MLFLOW_TRACKING_URI as str and the result of urllib.urlparse() of this infinstor tracking URI
    """
    ##########
    #  TODO: a copy exists in 
    #  infinstor-mlflow/plugin/infinstor_mlflow_plugin/login.py 
    #  infinstor-mlflow/processors/singlevm/scripts/rclocal.py
    #  infinstor-jupyterlab/server-extension/jupyterlab_infinstor/__init__.py
    #  infinstor-jupyterlab/server-extension/jupyterlab_infinstor/cognito_utils.py
    #  infinstor-jupyterlab/clientlib/infinstor/bootstrap.py
    #  infinstor-jupyterlab/infinstor_py_bootstrap_project/infinstor_py_bootstrap/infinstor_py_bootstrap.py
    #  Need to see how to share code between two pypi packages to eliminate this duplication
    #  when refactoring this code, also refactor the copies
    ############

    # Note that there are 3 ways to inject MLFLOW_TRACKING_URI: mlflow.set_tracking_uri(), MlflowClient(tracking_uri=xxxx) and MLFLOW_TRACKING_URI env variable.  
    # all 3 are used below (tracking_uri argument, set by caller CognitoAuthenticatedRestStore::__init__(), is derived MlflowClient(tracking_uri=xxxx) ).
    muri:str = tracking_uri
    pmuri:urllib.parse.ParseResult = urllib.parse.urlparse(muri)   # even if tracking_uri is None, a pmuri is returned. No exceptions raised
    if pmuri.scheme.lower() != 'infinstor':
        muri = mlflow.get_tracking_uri()
        pmuri = urllib.parse.urlparse(muri)
        if pmuri.scheme.lower() != 'infinstor':
            muri = os.getenv('MLFLOW_TRACKING_URI')
            pmuri = urllib.parse.urlparse(muri)
            if (pmuri.scheme.lower() != 'infinstor' and do_exit):
                # do not raise an exception since we want to display a user friendly error message. Printing the stack trace is not user friendly.  So print message and exit() instead
                #raise Exception("Error: MLFLOW_TRACKING_URI must be set to infinstor://.... to bootstrap infinstor mlflow client.  Please check documentation for details. Set the environment variable and retry.")
                _logger.error(f"Error: 'infinstor' scheme (for example infinstor://mlflow.infinstor.com) not detected for MLFLOW_TRACKING_URI: mlflow.set_tracking_uri()={mlflow.get_tracking_uri()} or MlflowClient(tracking_uri)={tracking_uri} or MLFLOW_TRACKING_URI environment variable={os.getenv('MLFLOW_TRACKING_URI')}.  Set one of these to infinstor scheme and retry.  Check documentation for details.")
                exit(1)
    
    return muri, pmuri
        

